@@ -1,19 +1,41 @@
 """Lights on Zigbee Home Automation networks."""
+from collections import Counter
 from datetime import timedelta
 import functools
+import itertools
 import logging
 import random
+from typing import Any, Dict, List, Optional, Tuple
 
+from zigpy.zcl.clusters.general import Identify, LevelControl, OnOff
+from zigpy.zcl.clusters.lighting import Color
 from zigpy.zcl.foundation import Status
 
 from homeassistant.components import light
-from homeassistant.const import STATE_ON
-from homeassistant.core import callback
+from homeassistant.components.light import (
+    ATTR_BRIGHTNESS,
+    ATTR_COLOR_TEMP,
+    ATTR_EFFECT,
+    ATTR_EFFECT_LIST,
+    ATTR_HS_COLOR,
+    ATTR_MAX_MIREDS,
+    ATTR_MIN_MIREDS,
+    ATTR_WHITE_VALUE,
+    SUPPORT_BRIGHTNESS,
+    SUPPORT_COLOR,
+    SUPPORT_COLOR_TEMP,
+    SUPPORT_EFFECT,
+    SUPPORT_FLASH,
+    SUPPORT_TRANSITION,
+    SUPPORT_WHITE_VALUE,
+)
+from homeassistant.const import ATTR_SUPPORTED_FEATURES, STATE_ON, STATE_UNAVAILABLE
+from homeassistant.core import State, callback
 from homeassistant.helpers.dispatcher import async_dispatcher_connect
 from homeassistant.helpers.event import async_track_time_interval
 import homeassistant.util.color as color_util
 
-from .core import discovery
+from .core import discovery, helpers
 from .core.const import (
     CHANNEL_COLOR,
     CHANNEL_LEVEL,
@@ -27,9 +49,10 @@ from .core.const import (
     SIGNAL_ATTR_UPDATED,
     SIGNAL_SET_LEVEL,
 )
+from .core.helpers import LogMixin
 from .core.registries import ZHA_ENTITIES
 from .core.typing import ZhaDeviceType
-from .entity import ZhaEntity
+from .entity import ZhaEntity, ZhaGroupEntity
 
 _LOGGER = logging.getLogger(__name__)
 
@@ -46,13 +69,23 @@ FLASH_EFFECTS = {light.FLASH_SHORT: EFFECT_BLINK, light.FLASH_LONG: EFFECT_BREAT
 
 UNSUPPORTED_ATTRIBUTE = 0x86
 STRICT_MATCH = functools.partial(ZHA_ENTITIES.strict_match, light.DOMAIN)
+GROUP_MATCH = functools.partial(ZHA_ENTITIES.group_match, light.DOMAIN)
 PARALLEL_UPDATES = 0
-_REFRESH_INTERVAL = (45, 75)
+
+SUPPORT_GROUP_LIGHT = (
+    SUPPORT_BRIGHTNESS
+    | SUPPORT_COLOR_TEMP
+    | SUPPORT_EFFECT
+    | SUPPORT_FLASH
+    | SUPPORT_COLOR
+    | SUPPORT_TRANSITION
+    | SUPPORT_WHITE_VALUE
+)
 
 
 async def async_setup_entry(hass, config_entry, async_add_entities):
     """Set up the Zigbee Home Automation light from config entry."""
-    entities_to_create = hass.data[DATA_ZHA][light.DOMAIN] = []
+    entities_to_create = hass.data[DATA_ZHA][light.DOMAIN]
 
     unsub = async_dispatcher_connect(
         hass,
@@ -64,46 +97,34 @@ async def async_setup_entry(hass, config_entry, async_add_entities):
     hass.data[DATA_ZHA][DATA_ZHA_DISPATCHERS].append(unsub)
 
 
-@STRICT_MATCH(channel_names=CHANNEL_ON_OFF, aux_channels={CHANNEL_COLOR, CHANNEL_LEVEL})
-class Light(ZhaEntity, light.Light):
-    """Representation of a ZHA or ZLL light."""
+class BaseLight(LogMixin, light.Light):
+    """Operations common to all light entities."""
 
-    def __init__(self, unique_id, zha_device: ZhaDeviceType, channels, **kwargs):
-        """Initialize the ZHA light."""
-        super().__init__(unique_id, zha_device, channels, **kwargs)
-        self._supported_features = 0
-        self._color_temp = None
-        self._hs_color = None
-        self._brightness = None
-        self._off_brightness = None
-        self._effect_list = []
-        self._effect = None
-        self._on_off_channel = self.cluster_channels.get(CHANNEL_ON_OFF)
-        self._level_channel = self.cluster_channels.get(CHANNEL_LEVEL)
-        self._color_channel = self.cluster_channels.get(CHANNEL_COLOR)
-        self._identify_channel = self.zha_device.channels.identify_ch
-        self._cancel_refresh_handle = None
+    def __init__(self, *args, **kwargs):
+        """Initialize the light."""
+        super().__init__(*args, **kwargs)
+        self._available: bool = False
+        self._brightness: Optional[int] = None
+        self._off_brightness: Optional[int] = None
+        self._hs_color: Optional[Tuple[float, float]] = None
+        self._color_temp: Optional[int] = None
+        self._min_mireds: Optional[int] = 154
+        self._max_mireds: Optional[int] = 500
+        self._white_value: Optional[int] = None
+        self._effect_list: Optional[List[str]] = None
+        self._effect: Optional[str] = None
+        self._supported_features: int = 0
+        self._state: bool = False
+        self._on_off_channel = None
+        self._level_channel = None
+        self._color_channel = None
+        self._identify_channel = None
 
-        if self._level_channel:
-            self._supported_features |= light.SUPPORT_BRIGHTNESS
-            self._supported_features |= light.SUPPORT_TRANSITION
-            self._brightness = 0
-
-        if self._color_channel:
-            color_capabilities = self._color_channel.get_color_capabilities()
-            if color_capabilities & CAPABILITIES_COLOR_TEMP:
-                self._supported_features |= light.SUPPORT_COLOR_TEMP
-
-            if color_capabilities & CAPABILITIES_COLOR_XY:
-                self._supported_features |= light.SUPPORT_COLOR
-                self._hs_color = (0, 0)
-
-            if color_capabilities & CAPABILITIES_COLOR_LOOP:
-                self._supported_features |= light.SUPPORT_EFFECT
-                self._effect_list.append(light.EFFECT_COLORLOOP)
-
-        if self._identify_channel:
-            self._supported_features |= light.SUPPORT_FLASH
+    @property
+    def device_state_attributes(self) -> Dict[str, Any]:
+        """Return state attributes."""
+        attributes = {"off_brightness": self._off_brightness}
+        return attributes
 
     @property
     def is_on(self) -> bool:
@@ -117,12 +138,6 @@ class Light(ZhaEntity, light.Light):
         """Return the brightness of this light."""
         return self._brightness
 
-    @property
-    def device_state_attributes(self):
-        """Return state attributes."""
-        attributes = {"off_brightness": self._off_brightness}
-        return attributes
-
     def set_level(self, value):
         """Set the brightness of this light between 0..254.
 
@@ -159,49 +174,6 @@ class Light(ZhaEntity, light.Light):
         """Flag supported features."""
         return self._supported_features
 
-    @callback
-    def async_set_state(self, attr_id, attr_name, value):
-        """Set the state."""
-        self._state = bool(value)
-        if value:
-            self._off_brightness = None
-        self.async_write_ha_state()
-
-    async def async_added_to_hass(self):
-        """Run when about to be added to hass."""
-        await super().async_added_to_hass()
-        await self.async_accept_signal(
-            self._on_off_channel, SIGNAL_ATTR_UPDATED, self.async_set_state
-        )
-        if self._level_channel:
-            await self.async_accept_signal(
-                self._level_channel, SIGNAL_SET_LEVEL, self.set_level
-            )
-        refresh_interval = random.randint(*_REFRESH_INTERVAL)
-        self._cancel_refresh_handle = async_track_time_interval(
-            self.hass, self._refresh, timedelta(minutes=refresh_interval)
-        )
-
-    async def async_will_remove_from_hass(self) -> None:
-        """Disconnect entity object when removed."""
-        self._cancel_refresh_handle()
-        await super().async_will_remove_from_hass()
-
-    @callback
-    def async_restore_last_state(self, last_state):
-        """Restore previous state."""
-        self._state = last_state.state == STATE_ON
-        if "brightness" in last_state.attributes:
-            self._brightness = last_state.attributes["brightness"]
-        if "off_brightness" in last_state.attributes:
-            self._off_brightness = last_state.attributes["off_brightness"]
-        if "color_temp" in last_state.attributes:
-            self._color_temp = last_state.attributes["color_temp"]
-        if "hs_color" in last_state.attributes:
-            self._hs_color = last_state.attributes["hs_color"]
-        if "effect" in last_state.attributes:
-            self._effect = last_state.attributes["effect"]
-
     async def async_turn_on(self, **kwargs):
         """Turn the entity on."""
         transition = kwargs.get(light.ATTR_TRANSITION)
@@ -330,6 +302,90 @@ class Light(ZhaEntity, light.Light):
 
         self.async_write_ha_state()
 
+
+@STRICT_MATCH(channel_names=CHANNEL_ON_OFF, aux_channels={CHANNEL_COLOR, CHANNEL_LEVEL})
+class Light(BaseLight, ZhaEntity):
+    """Representation of a ZHA or ZLL light."""
+
+    _REFRESH_INTERVAL = (45, 75)
+
+    def __init__(self, unique_id, zha_device: ZhaDeviceType, channels, **kwargs):
+        """Initialize the ZHA light."""
+        super().__init__(unique_id, zha_device, channels, **kwargs)
+        self._on_off_channel = self.cluster_channels.get(CHANNEL_ON_OFF)
+        self._level_channel = self.cluster_channels.get(CHANNEL_LEVEL)
+        self._color_channel = self.cluster_channels.get(CHANNEL_COLOR)
+        self._identify_channel = self.zha_device.channels.identify_ch
+        self._cancel_refresh_handle = None
+        effect_list = []
+
+        if self._level_channel:
+            self._supported_features |= light.SUPPORT_BRIGHTNESS
+            self._supported_features |= light.SUPPORT_TRANSITION
+            self._brightness = 0
+
+        if self._color_channel:
+            color_capabilities = self._color_channel.get_color_capabilities()
+            if color_capabilities & CAPABILITIES_COLOR_TEMP:
+                self._supported_features |= light.SUPPORT_COLOR_TEMP
+
+            if color_capabilities & CAPABILITIES_COLOR_XY:
+                self._supported_features |= light.SUPPORT_COLOR
+                self._hs_color = (0, 0)
+
+            if color_capabilities & CAPABILITIES_COLOR_LOOP:
+                self._supported_features |= light.SUPPORT_EFFECT
+                effect_list.append(light.EFFECT_COLORLOOP)
+
+        if self._identify_channel:
+            self._supported_features |= light.SUPPORT_FLASH
+
+        if effect_list:
+            self._effect_list = effect_list
+
+    @callback
+    def async_set_state(self, attr_id, attr_name, value):
+        """Set the state."""
+        self._state = bool(value)
+        if value:
+            self._off_brightness = None
+        self.async_write_ha_state()
+
+    async def async_added_to_hass(self):
+        """Run when about to be added to hass."""
+        await super().async_added_to_hass()
+        await self.async_accept_signal(
+            self._on_off_channel, SIGNAL_ATTR_UPDATED, self.async_set_state
+        )
+        if self._level_channel:
+            await self.async_accept_signal(
+                self._level_channel, SIGNAL_SET_LEVEL, self.set_level
+            )
+        refresh_interval = random.randint(*[x * 60 for x in self._REFRESH_INTERVAL])
+        self._cancel_refresh_handle = async_track_time_interval(
+            self.hass, self._refresh, timedelta(seconds=refresh_interval)
+        )
+
+    async def async_will_remove_from_hass(self) -> None:
+        """Disconnect entity object when removed."""
+        self._cancel_refresh_handle()
+        await super().async_will_remove_from_hass()
+
+    @callback
+    def async_restore_last_state(self, last_state):
+        """Restore previous state."""
+        self._state = last_state.state == STATE_ON
+        if "brightness" in last_state.attributes:
+            self._brightness = last_state.attributes["brightness"]
+        if "off_brightness" in last_state.attributes:
+            self._off_brightness = last_state.attributes["off_brightness"]
+        if "color_temp" in last_state.attributes:
+            self._color_temp = last_state.attributes["color_temp"]
+        if "hs_color" in last_state.attributes:
+            self._hs_color = last_state.attributes["hs_color"]
+        if "effect" in last_state.attributes:
+            self._effect = last_state.attributes["effect"]
+
     async def async_update(self):
         """Attempt to retrieve on off state from the light."""
         await super().async_update()
@@ -398,3 +454,77 @@ class Light(ZhaEntity, light.Light):
         """Call async_get_state at an interval."""
         await self.async_get_state(from_cache=False)
         self.async_write_ha_state()
+
+
+@STRICT_MATCH(
+    channel_names=CHANNEL_ON_OFF,
+    aux_channels={CHANNEL_COLOR, CHANNEL_LEVEL},
+    manufacturers="Philips",
+)
+class HueLight(Light):
+    """Representation of a HUE light which does not report attributes."""
+
+    _REFRESH_INTERVAL = (3, 5)
+
+
+@GROUP_MATCH()
+class LightGroup(BaseLight, ZhaGroupEntity):
+    """Representation of a light group."""
+
+    def __init__(
+        self, entity_ids: List[str], unique_id: str, group_id: int, zha_device, **kwargs
+    ) -> None:
+        """Initialize a light group."""
+        super().__init__(entity_ids, unique_id, group_id, zha_device, **kwargs)
+        group = self.zha_device.gateway.get_group(self._group_id)
+        self._on_off_channel = group.endpoint[OnOff.cluster_id]
+        self._level_channel = group.endpoint[LevelControl.cluster_id]
+        self._color_channel = group.endpoint[Color.cluster_id]
+        self._identify_channel = group.endpoint[Identify.cluster_id]
+
+    async def async_update(self) -> None:
+        """Query all members and determine the light group state."""
+        all_states = [self.hass.states.get(x) for x in self._entity_ids]
+        states: List[State] = list(filter(None, all_states))
+        on_states = [state for state in states if state.state == STATE_ON]
+
+        self._state = len(on_states) > 0
+        self._available = any(state.state != STATE_UNAVAILABLE for state in states)
+
+        self._brightness = helpers.reduce_attribute(on_states, ATTR_BRIGHTNESS)
+
+        self._hs_color = helpers.reduce_attribute(
+            on_states, ATTR_HS_COLOR, reduce=helpers.mean_tuple
+        )
+
+        self._white_value = helpers.reduce_attribute(on_states, ATTR_WHITE_VALUE)
+
+        self._color_temp = helpers.reduce_attribute(on_states, ATTR_COLOR_TEMP)
+        self._min_mireds = helpers.reduce_attribute(
+            states, ATTR_MIN_MIREDS, default=154, reduce=min
+        )
+        self._max_mireds = helpers.reduce_attribute(
+            states, ATTR_MAX_MIREDS, default=500, reduce=max
+        )
+
+        self._effect_list = None
+        all_effect_lists = list(helpers.find_state_attributes(states, ATTR_EFFECT_LIST))
+        if all_effect_lists:
+            # Merge all effects from all effect_lists with a union merge.
+            self._effect_list = list(set().union(*all_effect_lists))
+
+        self._effect = None
+        all_effects = list(helpers.find_state_attributes(on_states, ATTR_EFFECT))
+        if all_effects:
+            # Report the most common effect.
+            effects_count = Counter(itertools.chain(all_effects))
+            self._effect = effects_count.most_common(1)[0][0]
+
+        self._supported_features = 0
+        for support in helpers.find_state_attributes(states, ATTR_SUPPORTED_FEATURES):
+            # Merge supported features by emulating support for every feature
+            # we find.
+            self._supported_features |= support
+        # Bitwise-and the supported features with the GroupedLight's features
+        # so that we don't break in the future when a new feature is added.
+        self._supported_features &= SUPPORT_GROUP_LIGHT
